"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigLoader = void 0;
/*
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const fs_1 = require("fs");
const _ = require("lodash");
const path_1 = require("path");
const symbol_sdk_1 = require("symbol-sdk");
const logger_1 = require("../logger");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const BootstrapUtils_1 = require("./BootstrapUtils");
const logger = LoggerFactory_1.default.getLogger(logger_1.LogType.System);
class ConfigLoader {
    async generateRandomConfiguration(presetData) {
        const networkType = presetData.networkType;
        const addresses = {
            version: this.getAddressesMigration(presetData.networkType).length + 1,
            networkType: networkType,
            nemesisGenerationHashSeed: presetData.nemesisGenerationHashSeed || symbol_sdk_1.Account.generateNewAccount(networkType).publicAccount.publicKey,
        };
        if (presetData.nodes) {
            addresses.nodes = await this.generateNodeAccounts(presetData, networkType);
        }
        const sinkAddress = symbol_sdk_1.Account.generateNewAccount(networkType).address.plain();
        if (!presetData.harvestNetworkFeeSinkAddress) {
            presetData.harvestNetworkFeeSinkAddress = sinkAddress;
        }
        if (!presetData.mosaicRentalFeeSinkAddress) {
            presetData.mosaicRentalFeeSinkAddress = sinkAddress;
        }
        if (!presetData.namespaceRentalFeeSinkAddress) {
            presetData.namespaceRentalFeeSinkAddress = sinkAddress;
        }
        presetData.networkIdentifier = BootstrapUtils_1.BootstrapUtils.getNetworkIdentifier(presetData.networkType);
        presetData.networkName = BootstrapUtils_1.BootstrapUtils.getNetworkName(presetData.networkType);
        if (!presetData.nemesisGenerationHashSeed) {
            presetData.nemesisGenerationHashSeed = addresses.nemesisGenerationHashSeed;
        }
        if (presetData.nemesis) {
            addresses.nemesisSigner = ConfigLoader.toConfig(this.generateAccount(networkType, presetData.nemesis.nemesisSignerPrivateKey));
            if (!presetData.nemesis.nemesisSignerPrivateKey && addresses.nemesisSigner) {
                presetData.nemesis.nemesisSignerPrivateKey = addresses.nemesisSigner.privateKey;
            }
        }
        if (!presetData.nemesisSignerPublicKey && addresses.nemesisSigner) {
            presetData.nemesisSignerPublicKey = addresses.nemesisSigner.publicKey;
        }
        const nemesisSignerAddress = symbol_sdk_1.Address.createFromPublicKey(presetData.nemesisSignerPublicKey, networkType);
        if (!presetData.currencyMosaicId)
            presetData.currencyMosaicId = BootstrapUtils_1.BootstrapUtils.toHex(symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(0), nemesisSignerAddress).toHex());
        if (!presetData.harvestingMosaicId) {
            if (!presetData.nemesis) {
                throw new Error('nemesis must be defined!');
            }
            if (presetData.nemesis.mosaics && presetData.nemesis.mosaics.length > 1) {
                presetData.harvestingMosaicId = BootstrapUtils_1.BootstrapUtils.toHex(symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(1), nemesisSignerAddress).toHex());
            }
            else {
                presetData.harvestingMosaicId = presetData.currencyMosaicId;
            }
        }
        if (presetData.nemesis) {
            if (presetData.nemesis.mosaics) {
                const mosaics = [];
                presetData.nemesis.mosaics.forEach((m, index) => {
                    const accounts = this.generateAddresses(networkType, m.accounts);
                    mosaics.push({
                        id: index ? presetData.currencyMosaicId : presetData.harvestingMosaicId,
                        name: m.name,
                        type: index ? 'harvest' : 'currency',
                        accounts,
                    });
                });
                presetData.nemesis.mosaics.forEach((m, index) => {
                    const accounts = mosaics[index].accounts;
                    if (!m.currencyDistributions) {
                        const caNodes = (addresses.nodes || []).filter((node) => node.main);
                        const totalAccounts = (m.accounts || 0) + caNodes.length;
                        const amountPerAccount = Math.floor(m.supply / totalAccounts);
                        m.currencyDistributions = [
                            ...accounts.map((a) => ({ address: a.address, amount: amountPerAccount })),
                            ...caNodes.map((n) => ({ address: n.main.address, amount: amountPerAccount })),
                        ];
                        if (m.currencyDistributions.length)
                            m.currencyDistributions[0].amount += m.supply - totalAccounts * amountPerAccount;
                    }
                    const supplied = m.currencyDistributions.map((d) => d.amount).reduce((a, b) => a + b, 0);
                    if (m.supply != supplied) {
                        throw new Error(`Invalid nemgen total supplied value, expected ${m.supply} but total is ${supplied}`);
                    }
                });
                addresses.mosaics = mosaics;
            }
        }
        return addresses;
    }
    generateAddresses(networkType, size) {
        return ConfigLoader.getArray(size).map(() => ConfigLoader.toConfig(symbol_sdk_1.Account.generateNewAccount(networkType)));
    }
    generateAccount(networkType, privateKey) {
        return privateKey ? symbol_sdk_1.Account.createFromPrivateKey(privateKey, networkType) : symbol_sdk_1.Account.generateNewAccount(networkType);
    }
    generateNodeAccount(presetData, index, nodePreset, networkType) {
        const name = nodePreset.name || `node-${index}`;
        const ca = ConfigLoader.toConfig(this.generateAccount(networkType, nodePreset.mainPrivateKey));
        const node = ConfigLoader.toConfig(this.generateAccount(networkType, nodePreset.transportPrivateKey));
        const friendlyName = nodePreset.friendlyName || ca.publicKey.substr(0, 7);
        const nodeAccount = { name, friendlyName, roles: nodePreset.roles, main: ca, transport: node };
        const useRemoteAccount = nodePreset.nodeUseRemoteAccount || presetData.nodeUseRemoteAccount;
        if (useRemoteAccount && (nodePreset.harvesting || nodePreset.voting))
            nodeAccount.remote = ConfigLoader.toConfig(this.generateAccount(networkType, nodePreset.remotePrivateKey));
        if (nodePreset.voting)
            nodeAccount.voting = ConfigLoader.toConfig(this.generateAccount(networkType, nodePreset.votingPrivateKey));
        if (nodePreset.harvesting)
            nodeAccount.vrf = ConfigLoader.toConfig(this.generateAccount(networkType, nodePreset.vrfPrivateKey));
        return nodeAccount;
    }
    async generateNodeAccounts(presetData, networkType) {
        return Promise.all(presetData.nodes.map((node, index) => this.generateNodeAccount(presetData, index, node, networkType)));
    }
    static getArray(size) {
        return [...Array(size).keys()];
    }
    createPresetData({ password, root, preset, assembly, customPreset, customPresetObject, }) {
        const sharedPreset = BootstrapUtils_1.BootstrapUtils.loadYaml(path_1.join(root, 'presets', 'shared.yml'), false);
        const networkPreset = BootstrapUtils_1.BootstrapUtils.loadYaml(`${root}/presets/${preset}/network.yml`, false);
        const assemblyPreset = assembly ? BootstrapUtils_1.BootstrapUtils.loadYaml(`${root}/presets/${preset}/assembly-${assembly}.yml`, false) : {};
        const customPresetFileObject = customPreset ? BootstrapUtils_1.BootstrapUtils.loadYaml(customPreset, password) : {};
        //Deep merge
        const presetData = _.merge(sharedPreset, networkPreset, assemblyPreset, customPresetFileObject, customPresetObject, { preset });
        if (!ConfigLoader.presetInfoLogged) {
            logger.info(`Generating config from preset ${preset}`);
            if (assembly) {
                logger.info(`Assembly preset ${assembly}`);
            }
            if (customPreset) {
                logger.info(`Custom preset file ${customPreset}`);
            }
        }
        ConfigLoader.presetInfoLogged = true;
        if (presetData.assemblies && !assembly) {
            throw new Error(`Preset ${preset} requires assembly (-a, --assembly option). Possible values are: ${presetData.assemblies}`);
        }
        const presetDataWithDynamicDefaults = Object.assign(Object.assign({ version: 1, preset: preset }, presetData), { nodes: this.dynamicDefaultNodeConfiguration(presetData.nodes) });
        return this.expandRepeat(presetDataWithDynamicDefaults);
    }
    dynamicDefaultNodeConfiguration(nodes) {
        return _.map(nodes || [], (node) => {
            const roles = this.resolveRoles(node);
            if (node.harvesting && node.api) {
                return Object.assign(Object.assign({ syncsource: true, filespooling: true, partialtransaction: true, openPort: true, sinkType: 'Async', enableSingleThreadPool: false, brokerOpenPort: true, addressextraction: true, enableAutoSyncCleanup: false, mongo: true, zeromq: true }, node), { roles });
            }
            if (node.harvesting) {
                return Object.assign(Object.assign({ sinkType: 'Sync', enableSingleThreadPool: true, addressextraction: false, mongo: false, zeromq: false, syncsource: true, filespooling: false, enableAutoSyncCleanup: true, partialtransaction: false }, node), { roles });
            }
            if (node.api) {
                return Object.assign(Object.assign({ sinkType: 'Async', syncsource: false, filespooling: true, partialtransaction: true, enableSingleThreadPool: false, addressextraction: true, mongo: true, zeromq: true, enableAutoSyncCleanup: false }, node), { roles });
            }
            throw new Error('A node must have at least one harvesting: true or api: true');
        });
    }
    resolveRoles(nodePreset) {
        if (nodePreset.roles) {
            return nodePreset.roles;
        }
        const roles = [];
        if (nodePreset.harvesting) {
            roles.push('Peer');
        }
        if (nodePreset.api) {
            roles.push('Api');
        }
        if (nodePreset.voting) {
            roles.push('Voting');
        }
        return roles.join(',');
    }
    static toConfig(account) {
        return {
            privateKey: account.privateKey,
            publicKey: account.publicAccount.publicKey,
            address: account.address.plain(),
        };
    }
    expandRepeat(presetData) {
        return Object.assign(Object.assign({}, presetData), { databases: this.expandServicesRepeat(presetData, presetData.databases || []), nodes: this.expandServicesRepeat(presetData, presetData.nodes || []), gateways: this.expandServicesRepeat(presetData, presetData.gateways || []), explorers: this.expandServicesRepeat(presetData, presetData.explorers || []), wallets: this.expandServicesRepeat(presetData, presetData.wallets || []), faucets: this.expandServicesRepeat(presetData, presetData.faucets || []), nemesis: this.applyValueTemplate(presetData, presetData.nemesis) });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    applyValueTemplate(context, value) {
        if (!value) {
            return value;
        }
        if (_.isArray(value)) {
            return this.expandServicesRepeat(context, value);
        }
        if (_.isObject(value)) {
            return _.mapValues(value, (v) => this.applyValueTemplate(Object.assign(Object.assign({}, context), value), v));
        }
        if (!_.isString(value)) {
            return value;
        }
        return BootstrapUtils_1.BootstrapUtils.runTemplate(value, context);
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    expandServicesRepeat(context, services) {
        return _.flatMap(services || [], (service) => {
            if (service.repeat === 0) {
                return [];
            }
            return _.range(service.repeat || 1).map((index) => {
                return _.omit(_.mapValues(service, (v) => this.applyValueTemplate(Object.assign(Object.assign(Object.assign({}, context), service), { $index: index }), v)), 'repeat');
            });
        });
    }
    loadExistingPresetDataIfPreset(target, password) {
        const generatedPresetLocation = this.getGeneratedPresetLocation(target);
        if (fs_1.existsSync(generatedPresetLocation)) {
            return BootstrapUtils_1.BootstrapUtils.loadYaml(generatedPresetLocation, password);
        }
        return undefined;
    }
    loadExistingPresetData(target, password) {
        const presetData = this.loadExistingPresetDataIfPreset(target, password);
        if (!presetData) {
            throw new Error(`The file ${this.getGeneratedPresetLocation(target)} doesn't exist. Have you executed the 'config' command?`);
        }
        return presetData;
    }
    loadExistingAddressesIfPreset(target, password) {
        const generatedAddressLocation = this.getGeneratedAddressLocation(target);
        if (fs_1.existsSync(generatedAddressLocation)) {
            const presetData = this.loadExistingPresetData(target, password);
            return this.migrateAddresses(BootstrapUtils_1.BootstrapUtils.loadYaml(generatedAddressLocation, password), presetData.networkType);
        }
        return undefined;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    migrateAddresses(addresses, networkType) {
        const migrations = this.getAddressesMigration(networkType);
        return BootstrapUtils_1.BootstrapUtils.migrate('addresses.yml', addresses, migrations);
    }
    getAddressesMigration(networkType) {
        const generateAccount = this.generateAccount;
        return [
            {
                description: 'Key names migration',
                migrate(from) {
                    (from.nodes || []).forEach((nodeAddresses) => {
                        var _a;
                        if (nodeAddresses.signing) {
                            nodeAddresses.main = nodeAddresses.signing;
                        }
                        else {
                            if (nodeAddresses.ssl) {
                                nodeAddresses.main = ConfigLoader.toConfig(symbol_sdk_1.Account.createFromPrivateKey(nodeAddresses.ssl.privateKey, networkType));
                            }
                        }
                        nodeAddresses.transport = ConfigLoader.toConfig(generateAccount(networkType, (_a = nodeAddresses === null || nodeAddresses === void 0 ? void 0 : nodeAddresses.node) === null || _a === void 0 ? void 0 : _a.privateKey));
                        delete nodeAddresses.node;
                        delete nodeAddresses.signing;
                        delete nodeAddresses.ssl;
                    });
                    return from;
                },
            },
        ];
    }
    loadExistingAddresses(target, password) {
        const addresses = this.loadExistingAddressesIfPreset(target, password);
        if (!addresses) {
            throw new Error(`The file ${this.getGeneratedAddressLocation(target)} doesn't exist. Have you executed the 'config' command?`);
        }
        return addresses;
    }
    getGeneratedPresetLocation(target) {
        return path_1.join(target, 'preset.yml');
    }
    getGeneratedAddressLocation(target) {
        return path_1.join(target, 'addresses.yml');
    }
}
exports.ConfigLoader = ConfigLoader;
ConfigLoader.presetInfoLogged = false;
