"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SupernodeService = void 0;
/*
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const AnnounceService_1 = require("./AnnounceService");
const BootstrapUtils_1 = require("./BootstrapUtils");
const ConfigLoader_1 = require("./ConfigLoader");
const logger = LoggerFactory_1.default.getLogger();
class SupernodeService {
    constructor(params) {
        this.params = params;
        this.configLoader = new ConfigLoader_1.ConfigLoader();
    }
    async enroll(passedPresetData, passedAddresses) {
        const presetData = passedPresetData !== null && passedPresetData !== void 0 ? passedPresetData : this.configLoader.loadExistingPresetData(this.params.target, this.params.password);
        const addresses = passedAddresses !== null && passedAddresses !== void 0 ? passedAddresses : this.configLoader.loadExistingAddresses(this.params.target, this.params.password);
        if (!presetData.supernodeControllerPublicKey) {
            logger.warn('This network does not have a supernode controller public key. Nodes cannot be registered.');
            return;
        }
        await new AnnounceService_1.AnnounceService().announce(this.params.url, this.params.maxFee, this.params.useKnownRestGateways, presetData, addresses, this, '1M+');
    }
    async createTransactions({ presetData, nodePreset, nodeAccount, deadline, maxFee, }) {
        const transactions = [];
        const networkType = presetData.networkType;
        if (!nodePreset.supernode) {
            logger.warn(`Node ${nodeAccount.name} hasn't been configured as supernode.`);
            return transactions;
        }
        if (!nodePreset.voting) {
            logger.warn(`Node ${nodeAccount.name} 'voting: true' custom preset flag wasn't provided!`);
            return transactions;
        }
        if (!presetData.supernodeControllerPublicKey) {
            return transactions;
        }
        const supernodeControllerAddress = symbol_sdk_1.PublicAccount.createFromPublicKey(presetData.supernodeControllerPublicKey, networkType).address;
        const agentPublicKey = nodeAccount.transport.publicKey;
        if (!agentPublicKey) {
            logger.warn(`Cannot resolve harvester public key of node ${nodeAccount.name}`);
            return transactions;
        }
        if (!nodePreset.host) {
            logger.warn(`Node ${nodeAccount.name} public host name hasn't been provided! Please use 'host: myNodeHost' custom preset param.`);
            return transactions;
        }
        const agentUrl = nodePreset.agentUrl || `https://${nodePreset.host}:7880`;
        const plainMessage = `enrol ${agentPublicKey} ${agentUrl}`;
        const message = symbol_sdk_1.PlainMessage.create(plainMessage);
        logger.info(`Creating enrolment transfer with message '${plainMessage}'`);
        const transaction = symbol_sdk_1.TransferTransaction.create(deadline, supernodeControllerAddress, [], message, networkType, maxFee);
        return [transaction];
    }
}
exports.SupernodeService = SupernodeService;
SupernodeService.defaultParams = {
    useKnownRestGateways: false,
    target: BootstrapUtils_1.BootstrapUtils.defaultTargetFolder,
    url: 'http://localhost:3000',
};
